
#include "statistics.h"

#include <float.h>
#include <math.h>
#include <algorithm>
#include <numeric> 
#undef NDEBUG
#include <cassert>



double sumOfVector(std::vector<double> vec) {
	return accumulate(vec.begin(), vec.end(), 0.0);
}

double meanOfVector(std::vector<double> vec) {
	return sumOfVector(vec) / vec.size();
}


double maxOfVector(std::vector<double> vec) {
	double max = -DBL_MAX;
	for (double x : vec) {
		max = std::max(max, x);
	}
	return max;
}


int argmaxOfVector(std::vector<double> vec) {
	double max = -DBL_MAX;
	int ii = -1;
	for (int i = 0; i < vec.size(); i++) {
		if (vec[i] > max) {
			max = vec[i];
			ii = i;
		}
	}
	return ii;
}


double minOfVector(std::vector<double> vec) {
	double min = DBL_MAX;
	for (double x : vec) {
		min = std::min(min, x);
	}
	return min;
}


int argminOfVector(std::vector<double> vec) {
	double min = DBL_MAX;
	int ii = -1;
	for (int i = 0; i < vec.size(); i++) {
		if (vec[i] < min) {
			min = vec[i];
			ii = i;
		}
	}
	return ii;
}



double varianceOfVector(std::vector<double> vec, double mean) {
	double accum = 0.0;
	std::for_each(std::begin(vec), std::end(vec), [&](const double d) {
		accum += (d - mean) * (d - mean);
	});
	return accum / vec.size();
}

double stddevOfVector(std::vector<double> vec, double mean) {
	return sqrt(varianceOfVector(vec, mean));
}


double rmsDiffOfVectors(std::vector<double> vec1, std::vector<double> vec2) {
	assert(vec1.size() == vec2.size());
	double accum = 0.0;
	for (int i = 0; i < vec1.size(); i++) {
		double diff = vec1[i] - vec2[i];
		accum += diff * diff;
	}
	return sqrt(accum / vec1.size());
}



double quantileOfVector(std::vector<double> vec, double q) {
	assert(q > 0.0  &&  q < 1.0);
	std::sort(vec.begin(), vec.end(), [](const double & a, const double & b) {
		return a < b;   
	});
	double x = q * (vec.size() - 1);
	int n0 = (int)floor(x);
	int n1 = (int)ceil(x);
	assert(n0 >= 0 && n0 < (int)vec.size());
	assert(n1 >= 0 && n1 < (int)vec.size());
	double xx = x - n0;
	return vec[n0] * (1.0 - xx) + vec[n1] * xx;
}



void normaliseProbabilities(std::vector<double> & vec) {
	double sum = sumOfVector(vec);
	assert(sum > 0.0);
	for (int i = 0; i < vec.size(); i++) {
		vec[i] /= sum;
	}
}
